package com.llw.libjava.io.file.image;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;

import javax.imageio.ImageIO;
import javax.imageio.stream.FileImageOutputStream;

import com.llw.libjava.commons.util.ByteUtil;
import com.llw.libjava.commons.util.ExceptionUtil;
import com.llw.libjava.io.file.image.color.GrayValueCalculator;
import com.llw.libjava.io.file.image.color.GrayValueCalculator1;
import com.llw.libjava.io.file.image.color.GrayValueCalculator2;
import com.llw.libjava.io.file.image.color.GrayValueCalculator3;
import com.llw.libjava.io.file.image.color.GrayValueCalculator4;
import com.llw.libjava.io.file.image.color.GrayValueCalculator5;
import com.llw.libjava.io.file.image.color.RGB;

public class ImageUtil {

	private ImageUtil() {
	}

	public static byte[] readImageFileAsByteArray(File imageFile) throws IOException {
		if (!imageFile.exists()) {
			ExceptionUtil.throwIllegalArgumentException("Image file does not exist! --" + imageFile);
		}
		if (!imageFile.isFile()) {
			ExceptionUtil.throwIllegalArgumentException("Image file is not an image file! --" + imageFile);
		}
		byte[] byteArray = null;
		InputStream inputStream = null;
		try {
			inputStream = new FileInputStream(imageFile);
			int imageLength = inputStream.available();
			byteArray = new byte[imageLength];
			inputStream.read(byteArray);
		} finally {
			if (inputStream != null) {
				inputStream.close();
			}
		}
		return byteArray;
	}

	public static String readImageFileAsHexString(File imageFile) throws IOException {
		byte[] byteArray = readImageFileAsByteArray(imageFile);
		String hexString = ByteUtil.transformByteArray2HexString(byteArray);
		return hexString;
	}

	public static void writeByteArrayAsImageFile(byte[] byteArray, File imageFile) throws IOException {
		if (byteArray == null || byteArray.length == 0) {
			ExceptionUtil.throwEmptyArgumentException();
		}
		if (imageFile.exists() && !imageFile.isFile()) {
			ExceptionUtil.throwIllegalArgumentException("Image file exists, but is not an image file! --" + imageFile);
		}
		FileImageOutputStream outputStream = null;
		try {
			outputStream = new FileImageOutputStream(imageFile);
			outputStream.write(byteArray);
			outputStream.flush();
		} finally {
			if (outputStream != null) {
				outputStream.close();
			}
		}
	}

	public static void writeHexStringAsImageFile(String hexString, File imageFile) throws IOException {
		byte[] byteArray = ByteUtil.transformHexString2ByteArray(hexString);
		writeByteArrayAsImageFile(byteArray, imageFile);
	}

	public static void transformColorized2Gray(GrayValueCalculator grayValueCalculator, File colorizedImageFile, File grayImageFile) throws IOException {
		if (colorizedImageFile.exists() && !colorizedImageFile.isFile()) {
			ExceptionUtil.throwIllegalArgumentException("Image file exists, but is not an image file! --" + colorizedImageFile);
		}
		BufferedImage colorizedBufferedImage = ImageIO.read(colorizedImageFile);
		int width = colorizedBufferedImage.getWidth();
		int height = colorizedBufferedImage.getHeight();
		BufferedImage grayBufferedImage = new BufferedImage(width, height, BufferedImage.TYPE_BYTE_GRAY);
		for (int w = 0; w < width; w++) {
			for (int h = 0; h < height; h++) {
				int colorizedValue = colorizedBufferedImage.getRGB(w, h);
				int red = (colorizedValue >> 16) & 0xFF;
				int green = (colorizedValue >> 8) & 0xFF;
				int blue = colorizedValue & 0xFF;
				RGB rgb = new RGB();
				rgb.setRed((byte) red);
				rgb.setGreen((byte) green);
				rgb.setBlue((byte) blue);
				int gray = grayValueCalculator.calculate(rgb).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
				int grayValue = 0 | (gray << 16) | (gray << 8) | gray;
				grayBufferedImage.setRGB(w, h, grayValue);
			}
		}
		ImageIO.write(grayBufferedImage, "JPEG", grayImageFile);
	}

	public static void main(String[] args) {
		/*
		 * 
		 */
		// String sourceImagePath = "D:\\Workspace\\Java\\AEye-JavaLibrary-IO\\test\\1-43041919260201003x_vis_20120830_1_1.jpg";
		// String targetImagePath = "D:\\Workspace\\Java\\AEye-JavaLibrary-IO\\test\\2-43041919260201003x_vis_20120830_1_1.jpg";
		// try {
		// String hexString = ImageUtil.readImageFileAsHexString(new File(sourceImagePath));
		// System.out.println(hexString);
		// ImageUtil.writeHexStringAsImageFile(hexString, new File(targetImagePath));
		// } catch (IOException e) {
		// e.printStackTrace();
		// }

		/*
		 * 
		 */
		String colorizedImageDirectory = "D:\\Workspace\\Java\\AEye-JavaLibrary-IO\\test\\";
		String colorizedImageName = "1-43041919260201003x_vis_20120830_1_1.jpg";
		try {
			ImageUtil.transformColorized2Gray(new GrayValueCalculator1(), new File(colorizedImageDirectory + colorizedImageName),
					new File(colorizedImageDirectory + "gray2-1-" + colorizedImageName));
			ImageUtil.transformColorized2Gray(new GrayValueCalculator2(), new File(colorizedImageDirectory + colorizedImageName),
					new File(colorizedImageDirectory + "gray2-2-" + colorizedImageName));
			ImageUtil.transformColorized2Gray(new GrayValueCalculator3(), new File(colorizedImageDirectory + colorizedImageName),
					new File(colorizedImageDirectory + "gray2-3-" + colorizedImageName));
			ImageUtil.transformColorized2Gray(new GrayValueCalculator4(), new File(colorizedImageDirectory + colorizedImageName),
					new File(colorizedImageDirectory + "gray2-4-" + colorizedImageName));
			ImageUtil.transformColorized2Gray(new GrayValueCalculator5(), new File(colorizedImageDirectory + colorizedImageName),
					new File(colorizedImageDirectory + "gray2-5-" + colorizedImageName));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
