package com.llw.libjava.io.stream;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

public class StreamTool {
	public static final int DEFAULT_STREAM_BUFFER_SIZE = 10485760;
	private static final int DEFAULT_SKIPPING_TRY_TIMES_MAX = 10;
	private static final int DEFAULT_SKIPPING_TRY_TIMES_MIN = 1;

	private StreamTool() {
	}

	public static void skip(InputStream inputStream, long skipBytes, int maxTryTimes) throws IOException {
		long skipBytesLeft = skipBytes;
		int tryTimesLimit = maxTryTimes < DEFAULT_SKIPPING_TRY_TIMES_MIN ? DEFAULT_SKIPPING_TRY_TIMES_MIN
				: (maxTryTimes > DEFAULT_SKIPPING_TRY_TIMES_MAX ? DEFAULT_SKIPPING_TRY_TIMES_MAX : maxTryTimes);
		int tryTimes = 0;
		while (skipBytesLeft > 0) {
			long actualBytes = inputStream.skip(skipBytesLeft);
			skipBytesLeft -= actualBytes;
			if (++tryTimes > tryTimesLimit) {
				throw new IOException("can not try skipping for too many times -- " + tryTimes);
			}
		}
		if (skipBytesLeft != 0) {
			throw new IOException("can not skip specific bytes -- " + skipBytes);
		}
	}

	public static byte[] readPartFromInputStream(InputStream inputStream, int partSize) throws IOException {
		return readPartFromInputStream(inputStream, partSize, DEFAULT_STREAM_BUFFER_SIZE);
	}

	public static byte[] readPartFromInputStream(InputStream inputStream, int partSize, int readInBufferSize) throws IOException {
		int bufferSize = calculateBufferSize4PartReading(partSize, readInBufferSize);
		byte[] buffer = new byte[bufferSize];
		int actualReadInSize = inputStream.read(buffer);
		if (actualReadInSize <= 0) {
			return null;
		}
		List<StreamSegment> streamSegmentList = new ArrayList<StreamSegment>();
		addStreamSegmentIntoList(streamSegmentList, buffer, actualReadInSize);
		int streamLength = actualReadInSize;
		bufferSize = calculateBufferSize4PartReading((partSize - streamLength), bufferSize);
		while ((streamLength < partSize) && ((actualReadInSize = inputStream.read(buffer, 0, bufferSize)) > 0)) {
			addStreamSegmentIntoList(streamSegmentList, buffer, actualReadInSize);
			streamLength += actualReadInSize;
			bufferSize = calculateBufferSize4PartReading((partSize - streamLength), bufferSize);
		}
		/*
		 * {streamLength} must be equals to {partSize} now
		 */
		byte[] streamBytes = transferStreamSegmentList2ByteArray(streamSegmentList, streamLength);
		return streamBytes;
	}

	private static void addStreamSegmentIntoList(List<StreamSegment> streamSegmentList, byte[] buffer, int streamSegmentSize) {
		StreamSegment streamSegment = buildStreamSegmentByBuffer(buffer, streamSegmentSize);
		streamSegmentList.add(streamSegment);
	}

	private static StreamSegment buildStreamSegmentByBuffer(byte[] buffer, int streamSegmentSize) {
		byte[] segment = new byte[streamSegmentSize];
		System.arraycopy(buffer, 0, segment, 0, streamSegmentSize);
		StreamSegment streamSegment = new ActualStreamSegment(segment);
		return streamSegment;
	}

	private static int calculateBufferSize4PartReading(int partSizeHasntReadIn, int readInBufferSize) {
		return partSizeHasntReadIn < readInBufferSize ? partSizeHasntReadIn : readInBufferSize;
	}

	public static byte[] readAllFromInputStream(InputStream inputStream) throws IOException {
		return readAllFromInputStream(inputStream, DEFAULT_STREAM_BUFFER_SIZE);
	}

	public static byte[] readAllFromInputStream(InputStream inputStream, int readInBufferSize) throws IOException {
		byte[] buffer = new byte[readInBufferSize];
		int streamLength = inputStream.read(buffer);
		if (streamLength <= 0) {
			return null;
		}
		List<StreamSegment> streamSegmentList = new ArrayList<StreamSegment>();
		addStreamSegmentIntoList(streamSegmentList, buffer, streamLength);
		streamLength += readBufferFromAllInputStream(inputStream, readInBufferSize, streamSegmentList);
		byte[] streamBytes = transferStreamSegmentList2ByteArray(streamSegmentList, streamLength);
		return streamBytes;
	}

	private static int readBufferFromAllInputStream(InputStream inputStream, int readInBufferSize, List<StreamSegment> streamSegmentList) throws IOException {
		byte[] buffer = new byte[readInBufferSize];
		int actualReadInSize = 0;
		int streamLength = 0;
		while ((actualReadInSize = inputStream.read(buffer)) > 0) {
			addStreamSegmentIntoList(streamSegmentList, buffer, actualReadInSize);
			streamLength += actualReadInSize;
		}
		return streamLength;
	}

	private static byte[] transferStreamSegmentList2ByteArray(List<StreamSegment> streamSegmentList, int streamLength) {
		byte[] streamBytes = new byte[streamLength];
		int streamOffset = 0;
		int streamSegmentListSize = streamSegmentList.size();
		for (int i = 0; i < streamSegmentListSize; i++) {
			StreamSegment streamSegment = streamSegmentList.get(i);
			byte[] buffer = streamSegment.getSegment();
			int bufferSize = streamSegment.getSegmentSize();
			System.arraycopy(buffer, 0, streamBytes, streamOffset, bufferSize);
			streamOffset += bufferSize;
		}
		return streamBytes;
	}

	public static void transferStreamI2O(InputStream inputStream, OutputStream outputStream) throws IOException {
		transferStreamI2O(inputStream, outputStream, DEFAULT_STREAM_BUFFER_SIZE);
	}

	public static void transferStreamI2O(InputStream inputStream, OutputStream outputStream, int bufferSize) throws IOException {
		byte[] buffer = new byte[bufferSize];
		int actualReadIn = 0;
		while ((actualReadIn = inputStream.read(buffer)) > 0) {
			outputStream.write(buffer, 0, actualReadIn);
			outputStream.flush();
		}
	}
}
